      SUBROUTINE SVD(C,U,S,VH,M,MMAX)
C
C The subroutine SVD calls the subroutine BIDIAG to convert the complex
C matrix C to real, positive, upper bidiagonal form. The diagonal elements
C are stored in the vector S and its upper diagonal is stored in
C the vector UD. An implicit QR iteration with Wilkinson shift is then
C performed on the bidiagonal matrix through a series of Givens rotations
C to chase a rogue element, RE, from the matrix. After each QR iteration,
C the upper diagonal element in the leading row is tested for negligibility.
C The matrix is deflated when this element is found negligible, to
C computational accuracy, compared to its smallest diagonal neighbour,
C establishing a new leading row K. Similarly, a new trailing row, N, is
C established as the first row below the leading row with an upper diagonal
C element that is neglible compared to its smallest diagonal neighbour.
C The final iteration is on a 2 x 2 matrix. The singular values are on
C the diagonal of the final matrix. The singular values and their
C asociated eigenvectors are then sorted so that they are in descending
C order down the diagonal of the final matrix. The Hermitian transposes of
C the accumulated matrices U and VH are taken to convert them to
C the matrices appearing in the SVD factorization.
C
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      DOUBLE COMPLEX C(MMAX,MMAX),U(MMAX,MMAX),VH(MMAX,MMAX),HOLD
      DIMENSION S(MMAX),UD(MMAX)
C Set maximum number of iterations.
      MAXI=100
      CALL BIDIAG(U,C,VH,M,MMAX)
C Put diagonal of real, positive bidiagonal matrix in S and its upper
C diagonal in UD.
      DO 10 I=1,M
        IP1=I+1 
        S(I)=DREAL(C(I,I))
        IF(IP1.GT.M) GO TO 10
        UD(I)=DREAL(C(I,IP1))
 10   CONTINUE
C Begin diagonalization of matrix B.
C Set initial values of iteration number, ITER, starting row, K, and
C trailing row, N, of working matrix.
      ITER=0
      K=1
      N=M
C Continue reduction of working matrix.
 11   CONTINUE
C Advance iteration counter.
      ITER=ITER+1
      KP1=K+1
      NM1=N-1
C Find Wilkinson shift.
      P=S(N)
      Q=S(NM1)
      R=UD(NM1)
      H1=DSQRT((P+Q)*(P+Q)+R*R)
      H2=DSQRT((P-Q)*(P-Q)+R*R)
      SIG1=(H1-H2)/2.D0
C Find first Givens rotation.
      T11=(S(K)-SIG1)*(S(K)+SIG1)
      T12=S(K)*UD(K)
      CMAG=DSQRT(T11*T11+T12*T12)
      CO=T11/CMAG
      SI=-T12/CMAG
C Update unitary matrix VH.
      DO 12 I=1,M
        HOLD=VH(I,K)
        VH(I,K)=CO*HOLD-SI*VH(I,KP1)
        VH(I,KP1)=SI*HOLD+CO*VH(I,KP1)
 12   CONTINUE
C Post-multiply bidiagonal matrix B by rotation.
      H1=CO*S(K)-SI*UD(K)
      H2=CO*UD(K)+SI*S(K)
      S(K)=H1
      UD(K)=H2
C Find rogue element RE.
      RE=-SI*S(KP1)
      S(KP1)=CO*S(KP1)
C Find remaining Givens rotations.
      DO 13 I=K,NM1
      IP1=I+1
      IP2=I+2
      CMAG=DSQRT(S(I)*S(I)+RE*RE)
      CO=S(I)/CMAG
      SI=-RE/CMAG
C Update unitary matrix U.
      DO 14 J=1,M
        HOLD=U(I,J)
        U(I,J)=CO*HOLD-SI*U(IP1,J)
        U(IP1,J)=SI*HOLD+CO*U(IP1,J)
 14   CONTINUE
C Pre-multiply matrix B by rotation.
      H1=CMAG
      H2=CO*UD(I)-SI*S(IP1)
      H4=CO*S(IP1)+SI*UD(I)
      S(I)=H1
      UD(I)=H2
      S(IP1)=H4
      IF(IP2.GT.N) GO TO 13
      RE=-SI*UD(IP1)
      UD(IP1)=CO*UD(IP1)
C Find next Givens rotation.
      CMAG=DSQRT(UD(I)*UD(I)+RE*RE)
      CO=UD(I)/CMAG
      SI=-RE/CMAG
C  Update unitary matrix VH.
      DO 15 J=1,M
        HOLD=VH(J,IP1)
        VH(J,IP1)=CO*HOLD-SI*VH(J,IP2)
        VH(J,IP2)=SI*HOLD+CO*VH(J,IP2)
 15   CONTINUE
C Post-multiply matrix B by rotation.
      H1=CMAG
      H2=CO*S(IP2)
      H3=CO*S(IP1)-SI*UD(IP1)
      H4=CO*UD(IP1)+SI*S(IP1)
      UD(I)=H1
      S(IP1)=H3
      UD(IP1)=H4
      RE=-SI*S(IP2)
      S(IP2)=H2
 13   CONTINUE
C Find leading row of working matrix.
      J=K
      MM1=M-1
      DO 16 I=K,MM1
        IP1=I+1
C Find smallest neighbouring diagonal element.
        SMIN=DMIN1(DABS(S(I)),DABS(S(IP1)))
C Compare to see if upper diagonal element is negligible to machine precision.
        SIZE=SMIN+DABS(UD(I))
        IF(SIZE.NE.SMIN) GO TO 17
        J=J+1
 16   CONTINUE
 17   CONTINUE
C Set new leading row number.
      K=J
C Test to see if diagonalization is complete.
      IF(K.EQ.M) GO TO 20
C Find trailing row of working matrix.
      J=K+1
      KKP1=K+1
      DO 18 I=KKP1,MM1
        IP1=I+1
C Find smallest neighbouring diagonal element.
        SMIN=DMIN1(DABS(S(I)),DABS(S(IP1)))
C Compare to see if upper diagonal element is negligible.
        SIZE=SMIN+DABS(UD(I))
        IF(SIZE.EQ.SMIN) GO TO 19
        J=J+1
 18   CONTINUE
 19   CONTINUE
C Set new trailing row number.
      N=J
C Test for failure to converge.
      NNM1=N-1
      IF(KKP1.NE.KP1.OR.NNM1.NE.NM1) ITER=0
      IF(ITER.EQ.MAXI) PAUSE 'Maximum iterations reached.'
      GO TO 11
 20   CONTINUE
C Take Hermitian transpose of U and VH to get their final forms.
      DO 21 I=1,M
        DO 21 J=1,I
          HOLD=U(I,J)
          U(I,J)=DCONJG(U(J,I))
          U(J,I)=DCONJG(HOLD)
 21   CONTINUE
      DO 22 I=1,M
        DO 22 J=1,I
          HOLD=VH(I,J)
          VH(I,J)=DCONJG(VH(J,I))
          VH(J,I)=DCONJG(HOLD)
 22   CONTINUE
C Sort singular values into descending order, exchange columns of U
C and rows of VH.
      N=1
 23   CONTINUE
      IMAX=N
      DO 24 I=N,M
        IF(S(I).GE.S(IMAX)) IMAX=I
 24   CONTINUE
C Exchange singular values.
      HOLD=S(N)
      S(N)=S(IMAX)
      S(IMAX)=HOLD
C Exchange columns of U and rows of VH.
      DO 25 I=1,M
        HOLD=U(I,N)
        U(I,N)=U(I,IMAX)
        U(I,IMAX)=HOLD
        HOLD=VH(N,I)
        VH(N,I)=VH(IMAX,I)
        VH(IMAX,I)=HOLD
 25   CONTINUE  
      N=N+1
      IF(N.LE.M) GO TO 23    
      RETURN    
      END
